"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const MarkupToHtml_1 = require("@joplin/renderer/MarkupToHtml");
describe('MarkupToHtml', () => {
    it('should strip markup', (() => __awaiter(void 0, void 0, void 0, function* () {
        const service = new MarkupToHtml_1.default();
        const testCases = {
            [MarkupToHtml_1.default.MARKUP_LANGUAGE_MARKDOWN]: [
                ['', ''],
                ['## hello', 'hello'],
                ['## hello **hello!**', 'hello hello!'],
                ['*hi!*', 'hi!'],
                ['Some `code` here', 'Some code here'],
                ['Some <s>html</s> here', 'Some html here'],
                ['Some &amp; here', 'Some & here'],
                ['Some & here', 'Some & here'],
                ['[![image alt](:/fe9ea7fa727e4375b2e7d8a1b873314d)](https://example.com)', ''],
            ],
            [MarkupToHtml_1.default.MARKUP_LANGUAGE_HTML]: [
                ['<h1>hello</h1>', 'hello'],
                ['Some <b>strong</b> text', 'Some strong text'],
                ['<b>M&amp;Ms</b>', 'M&Ms'],
                ['<style>BODY{margin:0;padding:0;background:#f0f0f0}</style>', ''],
            ],
        };
        for (const markup in testCases) {
            for (const t of testCases[markup]) {
                const input = t[0];
                const expected = t[1];
                const actual = service.stripMarkup(Number(markup), input);
                expect(actual).toBe(expected);
            }
        }
        expect(service.stripMarkup(1, 'one line\n\ntwo line', { collapseWhiteSpaces: true })).toBe('one line two line');
        expect(service.stripMarkup(1, 'one line    two line', { collapseWhiteSpaces: true })).toBe('one line two line');
        expect(service.stripMarkup(1, 'one line\n    two line', { collapseWhiteSpaces: true })).toBe('one line two line');
    })));
    test('should escape HTML in safe mode', () => __awaiter(void 0, void 0, void 0, function* () {
        const service = new MarkupToHtml_1.default({ isSafeMode: true });
        const testString = '</pre>.<b>Test</b>';
        const expectedOutput = {
            html: '<pre>&lt;/pre&gt;.&lt;b&gt;Test&lt;/b&gt;</pre>',
            cssStrings: [],
            pluginAssets: [],
        };
        expect(yield service.render(MarkupToHtml_1.MarkupLanguage.Html, testString, {}, {})).toMatchObject(expectedOutput);
        expect(yield service.render(MarkupToHtml_1.MarkupLanguage.Markdown, testString, {}, {})).toMatchObject(expectedOutput);
    }));
});
//# sourceMappingURL=MarkupToHtml.js.map