"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const RepositoryApi_1 = require("@joplin/lib/services/plugins/RepositoryApi");
const shim_1 = require("@joplin/lib/shim");
const test_utils_1 = require("@joplin/lib/testing/test-utils");
function newRepoApi() {
    return __awaiter(this, void 0, void 0, function* () {
        const repo = new RepositoryApi_1.default(`${test_utils_1.supportDir}/pluginRepo`, yield (0, test_utils_1.createTempDir)());
        yield repo.initialize();
        return repo;
    });
}
describe('services_plugins_RepositoryApi', () => {
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        yield (0, test_utils_1.setupDatabaseAndSynchronizer)(1);
        yield (0, test_utils_1.switchClient)(1);
    }));
    it('should get the manifests', (() => __awaiter(void 0, void 0, void 0, function* () {
        const api = yield newRepoApi();
        const manifests = yield api.manifests();
        expect(!!manifests.find(m => m.id === 'joplin.plugin.ambrt.backlinksToNote')).toBe(true);
        expect(!!manifests.find(m => m.id === 'org.joplinapp.plugins.ToggleSidebars')).toBe(true);
    })));
    it('should search', (() => __awaiter(void 0, void 0, void 0, function* () {
        const api = yield newRepoApi();
        {
            const results = yield api.search('to');
            expect(results.length).toBe(2);
            expect(!!results.find(m => m.id === 'joplin.plugin.ambrt.backlinksToNote')).toBe(true);
            expect(!!results.find(m => m.id === 'org.joplinapp.plugins.ToggleSidebars')).toBe(true);
        }
        {
            const results = yield api.search('backlink');
            expect(results.length).toBe(1);
            expect(!!results.find(m => m.id === 'joplin.plugin.ambrt.backlinksToNote')).toBe(true);
        }
    })));
    it('should download a plugin', (() => __awaiter(void 0, void 0, void 0, function* () {
        const api = yield newRepoApi();
        const pluginPath = yield api.downloadPlugin('org.joplinapp.plugins.ToggleSidebars');
        expect(yield shim_1.default.fsDriver().exists(pluginPath)).toBe(true);
    })));
    it('should tell if a plugin can be updated', (() => __awaiter(void 0, void 0, void 0, function* () {
        const api = yield newRepoApi();
        expect(yield api.pluginCanBeUpdated('org.joplinapp.plugins.ToggleSidebars', '1.0.0', '3.0.0')).toBe(true);
        expect(yield api.pluginCanBeUpdated('org.joplinapp.plugins.ToggleSidebars', '1.0.0', '1.0.0')).toBe(false);
        expect(yield api.pluginCanBeUpdated('org.joplinapp.plugins.ToggleSidebars', '1.0.2', '3.0.0')).toBe(false);
        expect(yield api.pluginCanBeUpdated('does.not.exist', '1.0.0', '3.0.0')).toBe(false);
    })));
});
//# sourceMappingURL=RepositoryApi.js.map