"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Setting_1 = require("@joplin/lib/models/Setting");
const test_utils_1 = require("@joplin/lib/testing/test-utils");
const Folder_1 = require("@joplin/lib/models/Folder");
const testUtils_1 = require("../../../testUtils");
describe('JoplinSettings', () => {
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        yield (0, test_utils_1.setupDatabaseAndSynchronizer)(1);
        yield (0, test_utils_1.switchClient)(1);
    }));
    afterEach(() => __awaiter(void 0, void 0, void 0, function* () {
        yield (0, test_utils_1.afterEachCleanUp)();
    }));
    test('should listen to setting change event', () => __awaiter(void 0, void 0, void 0, function* () {
        const service = (0, testUtils_1.newPluginService)();
        const pluginScript = (0, testUtils_1.newPluginScript)(`
			joplin.plugins.register({
				onStart: async function() {
					await joplin.settings.registerSettings({
						'myCustomSetting1': {
							value: 1,
							type: 1,
							public: true,
							label: 'My Custom Setting 1',
						},
						'myCustomSetting2': {
							value: 2,
							type: 1,
							public: true,
							label: 'My Custom Setting 2',
						}
					})

					joplin.settings.onChange((event) => {
						joplin.data.post(['folders'], null, { title: JSON.stringify(event.keys) });
					});
				},
			});
		`);
        const plugin = yield service.loadPluginFromJsBundle('', pluginScript);
        yield service.runPlugin(plugin);
        Setting_1.default.setValue('plugin-org.joplinapp.plugins.PluginTest.myCustomSetting1', 111);
        Setting_1.default.setValue('plugin-org.joplinapp.plugins.PluginTest.myCustomSetting2', 222);
        // Also change a global setting, to verify that the plugin doesn't get
        // notifications for non-plugin related events.
        Setting_1.default.setValue('locale', 'fr_FR');
        Setting_1.default.emitScheduledChangeEvent();
        yield (0, test_utils_1.waitForFolderCount)(1);
        const folder = (yield Folder_1.default.all())[0];
        const settingNames = JSON.parse(folder.title);
        settingNames.sort();
        expect(settingNames.join(',')).toBe('myCustomSetting1,myCustomSetting2');
        yield service.destroy();
    }));
    test('should allow registering multiple settings', () => __awaiter(void 0, void 0, void 0, function* () {
        const service = (0, testUtils_1.newPluginService)();
        const pluginScript = (0, testUtils_1.newPluginScript)(`
			joplin.plugins.register({
				onStart: async function() {
					await joplin.settings.registerSettings({
						'myCustomSetting1': {
							value: 1,
							type: 1,
							public: true,
							label: 'My Custom Setting 1',
						},
						'myCustomSetting2': {
							value: 2,
							type: 1,
							public: true,
							label: 'My Custom Setting 2',
						}
					})
				},
			});
		`);
        const plugin = yield service.loadPluginFromJsBundle('', pluginScript);
        yield service.runPlugin(plugin);
        expect(Setting_1.default.value('plugin-org.joplinapp.plugins.PluginTest.myCustomSetting1')).toBe(1);
        expect(Setting_1.default.value('plugin-org.joplinapp.plugins.PluginTest.myCustomSetting2')).toBe(2);
        yield service.destroy();
    }));
});
//# sourceMappingURL=JoplinSettings.js.map