"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Setting_1 = require("@joplin/lib/models/Setting");
const test_utils_1 = require("@joplin/lib/testing/test-utils");
const Note_1 = require("@joplin/lib/models/Note");
const Folder_1 = require("@joplin/lib/models/Folder");
const ItemChange_1 = require("@joplin/lib/models/ItemChange");
const testUtils_1 = require("../../../testUtils");
describe('JoplinWorkspace', () => {
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        yield (0, test_utils_1.setupDatabaseAndSynchronizer)(1);
        yield (0, test_utils_1.switchClient)(1);
    }));
    afterEach(() => __awaiter(void 0, void 0, void 0, function* () {
        yield (0, test_utils_1.afterEachCleanUp)();
    }));
    test('should listen to noteChange events', () => __awaiter(void 0, void 0, void 0, function* () {
        const appState = {
            selectedNoteIds: [],
        };
        const service = (0, testUtils_1.newPluginService)('1.4', {
            getState: () => {
                return appState;
            },
        });
        const pluginScript = (0, testUtils_1.newPluginScript)(`			
			joplin.plugins.register({
				onStart: async function() {
					await joplin.workspace.onNoteChange(async (event) => {
						await joplin.data.post(['folders'], null, { title: JSON.stringify(event) });
					});
				},
			});
		`);
        const note = yield Note_1.default.save({});
        appState.selectedNoteIds.push(note.id);
        yield ItemChange_1.default.waitForAllSaved();
        const plugin = yield service.loadPluginFromJsBundle('', pluginScript);
        yield service.runPlugin(plugin);
        yield Note_1.default.save({ id: note.id, body: 'testing' });
        yield ItemChange_1.default.waitForAllSaved();
        const folder = (yield Folder_1.default.all())[0];
        const result = JSON.parse(folder.title);
        expect(result.id).toBe(note.id);
        expect(result.event).toBe(ItemChange_1.default.TYPE_UPDATE);
        yield service.destroy();
    }));
    test('should return the selected folder', () => __awaiter(void 0, void 0, void 0, function* () {
        const service = (0, testUtils_1.newPluginService)();
        const pluginScript = (0, testUtils_1.newPluginScript)(`			
			joplin.plugins.register({
				onStart: async function() {
					const folder = await joplin.workspace.selectedFolder();
					await joplin.data.put(['folders', folder.id], null, { title: "changedtitle" });
				},
			});
		`);
        const folder = yield Folder_1.default.save({ title: 'folder' });
        Setting_1.default.setValue('activeFolderId', folder.id);
        const plugin = yield service.loadPluginFromJsBundle('', pluginScript);
        yield service.runPlugin(plugin);
        const modFolder = yield Folder_1.default.load(folder.id);
        expect(modFolder.title).toBe('changedtitle');
    }));
});
//# sourceMappingURL=JoplinWorkspace.js.map