"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const KeymapService_1 = require("@joplin/lib/services/KeymapService");
const test_utils_1 = require("@joplin/lib/testing/test-utils");
const testUtils_1 = require("../../../testUtils");
describe('JoplinViewMenuItem', () => {
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        yield (0, test_utils_1.setupDatabaseAndSynchronizer)(1);
        yield (0, test_utils_1.switchClient)(1);
    }));
    afterEach(() => __awaiter(void 0, void 0, void 0, function* () {
        yield (0, test_utils_1.afterEachCleanUp)();
    }));
    test('should register commands with the keymap service', () => __awaiter(void 0, void 0, void 0, function* () {
        const service = (0, testUtils_1.newPluginService)();
        KeymapService_1.default.instance().initialize();
        const pluginScript = (0, testUtils_1.newPluginScript)(`			
			joplin.plugins.register({
				onStart: async function() {
					await joplin.commands.register({
						name: 'testCommand1',
						label: 'My Test Command 1',
						iconName: 'fas fa-music',
						execute: async () => {},
					});

					await joplin.views.menuItems.create('myMenuItem1', 'testCommand1', 'tools', { accelerator: 'CmdOrCtrl+Alt+Shift+B' });

					await joplin.commands.register({
						name: 'testCommand2',
						label: 'My Test Command 2',
						iconName: 'fas fa-music',
						execute: async () => {},
					});

					await joplin.views.menuItems.create('myMenuItem2', 'testCommand2', 'tools');
				},
			});
		`);
        const plugin = yield service.loadPluginFromJsBundle('', pluginScript);
        yield service.runPlugin(plugin);
        const commandNames = KeymapService_1.default.instance().getCommandNames();
        expect(commandNames.includes('testCommand1')).toBe(true);
        expect(commandNames.includes('testCommand2')).toBe(true);
        yield service.destroy();
    }));
});
//# sourceMappingURL=JoplinViewMenuItem.js.map